@@ -28,8 +28,6 @@
 #define DEBUG_LEVEL 5
 #include <re_dbg.h>
 
-#define PEMBUF_SIZE 512
-
 enum {
 	CONN_TIMEOUT = 30000,
 	RECV_TIMEOUT = 60000,
@@ -50,9 +48,7 @@ struct http_cli {
 	struct mbuf *cert;
 	struct mbuf *key;
 	struct sa laddr;
-#ifdef HAVE_INET6
 	struct sa laddr6;
-#endif
 	size_t bufsize_max;
 };
 
@@ -297,6 +293,9 @@ static int req_recv(struct http_req *req, struct mbuf *mb, bool *last)
 		return 0;
 	}
 
+	tmr_start(&req->conn->tmr, req->cli->conf.recv_timeout,
+		  timeout_handler, req->conn);
+
 	while (mbuf_get_left(mb)) {
 
 		if (req->rx_len == 0) {
@@ -603,10 +602,8 @@ static int conn_connect(struct http_req *req)
 
 	if (sa_af(&conn->addr) == AF_INET)
 		laddr = &req->cli->laddr;
-#ifdef HAVE_INET6
 	else if (sa_af(&conn->addr) == AF_INET6)
 		laddr = &req->cli->laddr6;
-#endif
 
 	if (sa_isset(laddr, SA_ADDR)) {
 		sa_set_scopeid(&conn->addr, sa_scopeid(laddr));
@@ -880,16 +877,13 @@ int http_request(struct http_req **reqp, struct http_cli *cli, const char *met,
 			goto out;
 	}
 	else {
-#ifdef HAVE_INET6
 		struct sa tmp;
-#endif
 		err = dnsc_query(&req->dq, cli->dnsc, req->host,
 				 DNS_TYPE_A, DNS_CLASS_IN, true,
 				 query_handler, req);
 		if (err)
 			goto out;
 
-#ifdef HAVE_INET6
 		if (0 == net_default_source_addr_get(AF_INET6, &tmp)) {
 
 			err = dnsc_query(&req->dq6, cli->dnsc, req->host,
@@ -898,7 +892,6 @@ int http_request(struct http_req **reqp, struct http_cli *cli, const char *met,
 			if (err)
 				goto out;
 		}
-#endif
 	}
 
  out:
@@ -1108,8 +1101,9 @@ int http_client_set_cert(struct http_cli *cli, const char *path)
 
 /**
  * Set client certificate in PEM format
+ *
  * @param cli    HTTP Client
- * @param pem    Client certificate in PEM format
+ * @param pem    Client certificate as 0-terminated string in PEM format
  *
  * @return 0 for success, error code otherwise.
  */
@@ -1120,11 +1114,19 @@ int http_client_set_certpem(struct http_cli *cli, const char *pem)
 		return EINVAL;
 
 	cli->cert = mem_deref(cli->cert);
-	cli->cert = mbuf_alloc(PEMBUF_SIZE);
+	cli->cert = mbuf_alloc(strlen(pem));
 	return mbuf_write_str(cli->cert, pem);
 }
 
 
+/**
+ * Set client key
+ *
+ * @param cli   HTTP Client
+ * @param path  File path to client key
+ *
+ * @return 0 for success, error code otherwise.
+ */
 int http_client_set_key(struct http_cli *cli, const char *path)
 {
 	int err = 0;
@@ -1143,6 +1145,14 @@ int http_client_set_key(struct http_cli *cli, const char *path)
 }
 
 
+/**
+ * Set client key in PEM format
+ *
+ * @param cli    HTTP Client
+ * @param pem    Client key as 0-terminated string in PEM format
+ *
+ * @return 0 for success, error code otherwise.
+ */
 int http_client_set_keypem(struct http_cli *cli, const char *pem)
 {
 	if (!cli || !str_isset(pem))
@@ -1150,7 +1160,7 @@ int http_client_set_keypem(struct http_cli *cli, const char *pem)
 
 	cli->key = mem_deref(cli->key);
 
-	cli->key = mbuf_alloc(PEMBUF_SIZE);
+	cli->key = mbuf_alloc(strlen(pem));
 	return mbuf_write_str(cli->key, pem);
 }
 
@@ -1227,7 +1237,77 @@ int http_client_set_tls_max_version(struct http_cli *cli, int version)
 
 	return tls_set_max_proto_version(cli->tls, version);
 }
-#endif
+
+
+/**
+ * Disable TLS server certificate verification
+ *
+ * @param cli	HTTP Client
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int http_client_disable_verify_server(struct http_cli *cli)
+{
+	if (!cli)
+		return EINVAL;
+
+	tls_disable_verify_server(cli->tls);
+
+	return 0;
+}
+
+
+/**
+ * Change used certificate+key of TLS connection
+ *
+ * @param cli       HTTP Client
+ * @param chain     Cert (chain) + Key (PEM format)
+ * @param len_chain Length of certificate + key PEM string
+ *
+ * @return int 0 if success, otherwise errorcode
+ */
+int http_client_use_chainpem(struct http_cli *cli, const char *chain,
+			     int len_chain)
+{
+	if (!cli || !cli->tls)
+		return EINVAL;
+
+	int err = tls_set_certificate_chain_pem(cli->tls, chain, len_chain);
+	if (err)
+		return err;
+
+	cli->cert = mem_deref(cli->cert);
+	cli->key = mem_deref(cli->key);
+
+	return 0;
+}
+
+
+/**
+ * Change used certificate+key of TLS connection
+ *
+ * @param cli  HTTP Client
+ * @param path Path to Cert (chain) + Key file (PEM format)
+ *
+ * @return int 0 if success, otherwise errorcode
+ */
+int http_client_use_chain(struct http_cli *cli, const char *path)
+{
+	int err;
+
+	if (!cli || !cli->tls)
+		return EINVAL;
+
+	err =  tls_set_certificate_chain(cli->tls, path);
+	if (err)
+		return err;
+
+	cli->cert = mem_deref(cli->cert);
+	cli->key = mem_deref(cli->key);
+
+	return 0;
+}
+#endif /* USE_TLS */
 
 
 /**
@@ -1253,13 +1333,8 @@ void http_client_set_laddr(struct http_cli *cli, const struct sa *addr)
  */
 void http_client_set_laddr6(struct http_cli *cli, const struct sa *addr)
 {
-#ifdef HAVE_INET6
 	if (cli && addr)
 		sa_cpy(&cli->laddr6, addr);
-#else
-	(void)cli;
-	(void)addr;
-#endif
 }
 
 
