@@ -209,11 +209,8 @@ static int getaddr_rcand(void *arg)
 {
 	struct rcand *rcand = arg;
 	struct addrinfo *res, *res0 = NULL;
-#ifndef OPENBSD
-	struct addrinfo hints = {.ai_flags  = AI_V4MAPPED | AI_ADDRCONFIG,
-#else
+
 	struct addrinfo hints = {.ai_flags  = AI_ADDRCONFIG,
-#endif
 				 .ai_family = rcand->ai_family};
 	int err;
 
@@ -245,7 +242,8 @@ static void delayed_rcand(int err, void *arg)
 		goto out;
 
 	if (!rcand->icem->rcand_wait) {
-		DEBUG_WARNING("late mDNS candidate: %s\n", rcand->domain);
+		DEBUG_WARNING("late mDNS candidate: %s - %J\n", rcand->domain,
+			      rcand->caddr);
 		goto out;
 	}
 
