@@ -10,6 +10,7 @@
 #include <openssl/bn.h>
 #include <openssl/evp.h>
 #include <openssl/ec.h>
+#include <openssl/pem.h>
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 #include <re_types.h>
@@ -57,9 +58,7 @@ struct tls {
  */
 struct tls_cert {
 	struct le le;
-	X509 *x509;
-	EVP_PKEY *pkey;
-	STACK_OF(X509) *chain;
+	SSL_CTX *ctx;
 	char *host;
 };
 
@@ -150,20 +149,6 @@ static int password_cb(char *buf, int size, int rwflag, void *userdata)
 }
 
 
-static int keytype2int(enum tls_keytype type)
-{
-	switch (type) {
-	case TLS_KEYTYPE_EC:
-		return EVP_PKEY_EC;
-	case TLS_KEYTYPE_RSA:
-		return EVP_PKEY_RSA;
-	default:
-		return EVP_PKEY_NONE;
-	}
-}
-
-
-#if !defined(LIBRESSL_VERSION_NUMBER)
 /**
  * OpenSSL verify handler for debugging purposes. Prints only warnings in the
  * default build
@@ -173,7 +158,7 @@ static int keytype2int(enum tls_keytype type)
  *
  * @return passes parameter ok unchanged
  */
-int tls_verify_handler(int ok, X509_STORE_CTX *ctx)
+static int tls_verify_handler(int ok, X509_STORE_CTX *ctx)
 {
 	int err, depth;
 
@@ -206,7 +191,6 @@ int tls_verify_handler(int ok, X509_STORE_CTX *ctx)
 
 	return ok;
 }
-#endif
 
 
 static int tls_verify_idx = -1;
@@ -222,6 +206,80 @@ static void tls_init_verify_idx(void)
 }
 
 
+static int tls_ctx_alloc(SSL_CTX **ctxp, enum tls_method method,
+			 const char *certf, const char *pwd, struct tls *tls)
+{
+	int err = 0;
+	int r;
+	SSL_CTX *ctx;
+	int min_proto = 0;
+
+	switch (method) {
+
+	case TLS_METHOD_TLS:
+	case TLS_METHOD_SSLV23:
+		ctx	  = SSL_CTX_new(TLS_method());
+		min_proto = TLS1_2_VERSION;
+		break;
+
+	case TLS_METHOD_DTLS:
+	case TLS_METHOD_DTLSV1:
+	case TLS_METHOD_DTLSV1_2:
+		ctx = SSL_CTX_new(DTLS_method());
+		break;
+
+	default:
+		DEBUG_WARNING("tls method %d not supported\n", method);
+		return ENOSYS;
+	}
+
+	if (!ctx) {
+		ERR_clear_error();
+		return ENOMEM;
+	}
+
+	SSL_CTX_set_min_proto_version(ctx, min_proto);
+
+	if (!certf)
+		goto out;
+
+	/* Load our keys and certificates */
+	if (pwd && tls) {
+		err = str_dup(&tls->pass, pwd);
+		if (err)
+			goto out;
+
+		SSL_CTX_set_default_passwd_cb(ctx, password_cb);
+		SSL_CTX_set_default_passwd_cb_userdata(ctx, tls);
+	}
+
+	r = SSL_CTX_use_certificate_chain_file(ctx, certf);
+	if (r <= 0) {
+		DEBUG_WARNING("Can't read certificate file: %s (%d)\n", certf,
+			      r);
+		ERR_clear_error();
+		err = EINVAL;
+		goto out;
+	}
+
+	r = SSL_CTX_use_PrivateKey_file(ctx, certf, SSL_FILETYPE_PEM);
+	if (r <= 0) {
+		DEBUG_WARNING("Can't read key file: %s (%d)\n", certf, r);
+		ERR_clear_error();
+		err = EINVAL;
+		goto out;
+	}
+
+out:
+	if (err)
+		SSL_CTX_free(ctx);
+	else
+		*ctxp = ctx;
+
+	return err;
+}
+
+
 /**
  * Allocate a new TLS context
  *
@@ -236,8 +294,7 @@ int tls_alloc(struct tls **tlsp, enum tls_method method, const char *keyfile,
 	      const char *pwd)
 {
 	struct tls *tls;
-	int r, err;
-	int min_proto = 0;
+	int err;
 
 	if (!tlsp)
 		return EINVAL;
@@ -246,72 +303,16 @@ int tls_alloc(struct tls **tlsp, enum tls_method method, const char *keyfile,
 	if (!tls)
 		return ENOMEM;
 
-	tls->verify_server = true;
-	switch (method) {
-
-	case TLS_METHOD_TLS:
-	case TLS_METHOD_SSLV23:
-		tls->ctx = SSL_CTX_new(TLS_method());
-		min_proto = TLS1_2_VERSION;
-		break;
-
-	case TLS_METHOD_DTLS:
-	case TLS_METHOD_DTLSV1:
-	case TLS_METHOD_DTLSV1_2:
-		tls->ctx = SSL_CTX_new(DTLS_method());
-		break;
-
-	default:
-		DEBUG_WARNING("tls method %d not supported\n", method);
-		err = ENOSYS;
-		goto out;
-	}
-
-	if (!tls->ctx) {
-		ERR_clear_error();
-		err = ENOMEM;
-		goto out;
-	}
-
-	err = tls_set_min_proto_version(tls, min_proto);
+	err = tls_ctx_alloc(&tls->ctx, method, keyfile, pwd, tls);
 	if (err)
 		goto out;
 
+	tls->verify_server = true;
+
 #if defined(TRACE_SSL)
 	SSL_CTX_set_keylog_callback(tls->ctx, tls_keylogger_cb);
 #endif
 
-	/* Load our keys and certificates */
-	if (keyfile) {
-		if (pwd) {
-			err = str_dup(&tls->pass, pwd);
-			if (err)
-				goto out;
-
-			SSL_CTX_set_default_passwd_cb(tls->ctx, password_cb);
-			SSL_CTX_set_default_passwd_cb_userdata(tls->ctx, tls);
-		}
-
-		r = SSL_CTX_use_certificate_chain_file(tls->ctx, keyfile);
-		if (r <= 0) {
-			DEBUG_WARNING("Can't read certificate file: %s (%d)\n",
-				      keyfile, r);
-			ERR_clear_error();
-			err = EINVAL;
-			goto out;
-		}
-
-		r = SSL_CTX_use_PrivateKey_file(tls->ctx, keyfile,
-						SSL_FILETYPE_PEM);
-		if (r <= 0) {
-			DEBUG_WARNING("Can't read key file: %s (%d)\n",
-				      keyfile, r);
-			ERR_clear_error();
-			err = EINVAL;
-			goto out;
-		}
-	}
-
 	err = hash_alloc(&tls->reuse.ht_sessions, 256);
 	if (err)
 		goto out;
@@ -481,7 +482,7 @@ int tls_set_verify_purpose(struct tls *tls, const char *purpose)
 {
 	int err;
 	int i;
-	X509_PURPOSE *xptmp;
+	const X509_PURPOSE *xptmp;
 
 	if (!tls || !purpose)
 		return EINVAL;
@@ -555,7 +556,7 @@ static int tls_generate_cert(X509 **pcert, const char *cn)
 /**
  * Create a selfsigned X509 certificate using EC
  *
- * @param tls      TLS Contect
+ * @param tls      TLS Context
  * @param cn       Common Name
  * @param curve_n  Known EC curve name
  *
@@ -643,90 +644,6 @@ int tls_set_selfsigned_ec(struct tls *tls, const char *cn, const char *curve_n)
 }
 
 
-int tls_set_selfsigned_rsa(struct tls *tls, const char *cn, size_t bits)
-{
-	EVP_PKEY *key = NULL;
-	X509 *cert = NULL;
-#ifndef OPENSSL_VERSION_MAJOR
-	BIGNUM *bn = NULL;
-	RSA *rsa = NULL;
-#endif
-	int r, err = ENOMEM;
-
-	if (!tls || !cn)
-		return EINVAL;
-
-#if OPENSSL_VERSION_MAJOR >= 3
-	key = EVP_RSA_gen(bits);
-	if (!key)
-		goto out;
-#else
-	rsa = RSA_new();
-	if (!rsa)
-		goto out;
-
-	bn = BN_new();
-	if (!bn)
-		goto out;
-
-	BN_set_word(bn, RSA_F4);
-	if (!RSA_generate_key_ex(rsa, (int)bits, bn, NULL))
-		goto out;
-
-	key = EVP_PKEY_new();
-	if (!key)
-		goto out;
-
-	if (!EVP_PKEY_set1_RSA(key, rsa))
-		goto out;
-#endif
-
-	if (tls_generate_cert(&cert, cn))
-		goto out;
-
-	if (!X509_set_pubkey(cert, key))
-		goto out;
-
-	if (!X509_sign(cert, key, EVP_sha256()))
-		goto out;
-
-	r = SSL_CTX_use_certificate(tls->ctx, cert);
-	if (r != 1)
-		goto out;
-
-	r = SSL_CTX_use_PrivateKey(tls->ctx, key);
-	if (r != 1)
-		goto out;
-
-	if (tls->cert)
-		X509_free(tls->cert);
-
-	tls->cert = cert;
-	cert = NULL;
-
-	err = 0;
-
- out:
-	if (cert)
-		X509_free(cert);
-
-	if (key)
-		EVP_PKEY_free(key);
-
-#ifndef OPENSSL_VERSION_MAJOR
-	if (rsa)
-		RSA_free(rsa);
-
-	if (bn)
-		BN_free(bn);
-#endif
-
-	if (err)
-		ERR_clear_error();
-
-	return err;
-}
-
 /**
  * Set the certificate and private key on a TLS context
  *
@@ -848,79 +765,6 @@ int tls_set_certificate_pem(struct tls *tls, const char *cert, size_t len_cert,
 }
 
 
-/**
- * Set the certificate and private key on a TLS context
- *
- * @param tls      TLS Context
- * @param keytype  Private key type
- * @param cert     Certificate in DER format
- * @param len_cert Length of certificate DER bytes
- * @param key      Private key in DER format, will be read from cert if NULL
- * @param len_key  Length of private key DER bytes
- *
- * @return 0 if success, otherwise errorcode
- */
-int tls_set_certificate_der(struct tls *tls, enum tls_keytype keytype,
-			    const uint8_t *cert, size_t len_cert,
-			    const uint8_t *key, size_t len_key)
-{
-	const uint8_t *buf_cert;
-	X509 *x509 = NULL;
-	EVP_PKEY *pkey = NULL;
-	int r, type, err = ENOMEM;
-
-	if (!tls || !cert || !len_cert || (key && !len_key))
-		return EINVAL;
-
-	type = keytype2int(keytype);
-	if (type == EVP_PKEY_NONE)
-		return EINVAL;
-
-	buf_cert = cert;
-
-	x509 = d2i_X509(NULL, &buf_cert, (long)len_cert);
-	if (!x509)
-		goto out;
-
-	if (!key) {
-		key = buf_cert;
-		len_key = len_cert - (buf_cert - cert);
-	}
-
-	pkey = d2i_PrivateKey(type, NULL, &key, (long)len_key);
-	if (!pkey)
-		goto out;
-
-	r = SSL_CTX_use_certificate(tls->ctx, x509);
-	if (r != 1)
-		goto out;
-
-	r = SSL_CTX_use_PrivateKey(tls->ctx, pkey);
-	if (r != 1) {
-		DEBUG_WARNING("set_certificate_der: use_PrivateKey failed\n");
-		goto out;
-	}
-
-	if (tls->cert)
-		X509_free(tls->cert);
-
-	tls->cert = x509;
-	x509 = NULL;
-
-	err = 0;
-
- out:
-	if (x509)
-		X509_free(x509);
-	if (pkey)
-		EVP_PKEY_free(pkey);
-	if (err)
-		ERR_clear_error();
-
-	return err;
-}
-
-
 /**
  * Set the certificate and private key on a TLS context
  *
@@ -962,22 +806,6 @@ void tls_set_verify_client_trust_all(struct tls *tls)
 }
 
 
-/**
- * Set TLS server context to request certificate from peer
- * and set trust all certificates of peer.
- *
- * @deprecated Use tls_set_verify_peer_trust_all instead
- * @param tls    TLS Context
- */
-void tls_set_verify_client(struct tls *tls)
-{
-	if (!tls)
-		return;
-
-	tls_set_verify_client_trust_all(tls);
-}
-
-
 static int tls_verify_handler_ud(int ok, X509_STORE_CTX *ctx)
 {
 	int ret = ok;
@@ -1047,7 +875,6 @@ static int tls_verify_handler_ud(int ok, X509_STORE_CTX *ctx)
 int tls_set_verify_client_handler(struct tls_conn *tc, int depth,
 	int (*verifyh) (int ok, void *arg), void *arg)
 {
-#if !defined(LIBRESSL_VERSION_NUMBER)
 	int err = 0;
 	SSL_verify_cb tls_cb = tls_verify_handler_ud;
 	if (!tc)
@@ -1068,14 +895,6 @@ int tls_set_verify_client_handler(struct tls_conn *tc, int depth,
 		tls_cb);
 
 	return err;
-#else
-	(void) tc;
-	(void) depth;
-	(void) verifyh;
-	(void) arg;
-	(void) tls_verify_handler_ud;
-	return ENOSYS;
-#endif
 }
 
 
@@ -1089,7 +908,6 @@ int tls_set_verify_client_handler(struct tls_conn *tc, int depth,
  */
 int tls_set_srtp(struct tls *tls, const char *suites)
 {
-#ifdef USE_OPENSSL_SRTP
 	if (!tls || !suites)
 		return EINVAL;
 
@@ -1099,12 +917,6 @@ int tls_set_srtp(struct tls *tls, const char *suites)
 	}
 
 	return 0;
-#else
-	(void)tls;
-	(void)suites;
-
-	return ENOSYS;
-#endif
 }
 
 
@@ -1265,7 +1077,6 @@ int tls_srtp_keyinfo(const struct tls_conn *tc, enum srtp_suite *suite,
 		     uint8_t *cli_key, size_t cli_key_size,
 		     uint8_t *srv_key, size_t srv_key_size)
 {
-#ifdef USE_OPENSSL_SRTP
 	static const char *label = "EXTRACTOR-dtls_srtp";
 	size_t key_size, salt_size, size;
 	SRTP_PROTECTION_PROFILE *sel;
@@ -1336,16 +1147,6 @@ int tls_srtp_keyinfo(const struct tls_conn *tc, enum srtp_suite *suite,
 	mem_secclean(keymat, sizeof(keymat));
 
 	return 0;
-#else
-	(void)tc;
-	(void)suite;
-	(void)cli_key;
-	(void)cli_key_size;
-	(void)srv_key;
-	(void)srv_key_size;
-
-	return ENOSYS;
-#endif
 }
 
 
@@ -1422,7 +1223,6 @@ int tls_set_ciphers(struct tls *tls, const char *cipherv[], size_t count)
  */
 int tls_set_verify_server(struct tls_conn *tc, const char *host)
 {
-#if !defined(LIBRESSL_VERSION_NUMBER)
 	struct sa sa;
 
 	if (!tc || !host)
@@ -1451,12 +1251,6 @@ int tls_set_verify_server(struct tls_conn *tc, const char *host)
 	SSL_set_verify(tc->ssl, SSL_VERIFY_PEER, tls_verify_handler);
 
 	return 0;
-#else
-	(void)tc;
-	(void)host;
-
-	return ENOSYS;
-#endif
 }
 
 
@@ -1469,8 +1263,6 @@ int tls_set_verify_server(struct tls_conn *tc, const char *host)
  */
 int tls_verify_client(struct tls_conn *tc)
 {
-#if !defined(LIBRESSL_VERSION_NUMBER)
-
 	if (!tc)
 		return EINVAL;
 
@@ -1481,11 +1273,6 @@ int tls_verify_client(struct tls_conn *tc)
 		       tls_verify_handler);
 
 	return 0;
-#else
-	(void)tc;
-
-	return ENOSYS;
-#endif
 }
 
 
@@ -1733,10 +1520,7 @@ static int tls_session_update_cache(const struct tls_conn *tc,
 					     session_cmp_handler, &peer));
 	mem_deref(e);
 
-#if (OPENSSL_VERSION_NUMBER >= 0x10101000L) && \
-	(!defined(LIBRESSL_VERSION_NUMBER) || \
-	  defined(LIBRESSL_HAS_TLS1_3) || \
-	  defined(LIBRESSL_INTERNAL) )
+#if !defined(LIBRESSL_VERSION_NUMBER)
 	if (!SSL_SESSION_is_resumable(sess)) {
 		return EINVAL;
 	}
@@ -1759,7 +1543,6 @@ static int tls_session_update_cache(const struct tls_conn *tc,
 }
 
 
-#if (OPENSSL_VERSION_NUMBER >= 0x10101000L)
 static int session_new_cb(struct ssl_st *ssl, SSL_SESSION *sess)
 {
 	BIO *wbio = NULL;
@@ -1817,7 +1600,6 @@ static void session_remove_cb(SSL_CTX *ctx, SSL_SESSION *sess)
 	/* iterate over all hash table entries and search for session */
 	(void) hash_apply(tls->reuse.ht_sessions, remove_handler, sess);
 }
-#endif
 
 
 /**
@@ -1843,14 +1625,10 @@ int tls_set_session_reuse(struct tls *tls, int enabled)
 	if (!tls->reuse.enabled)
 		return 0;
 
-#if (OPENSSL_VERSION_NUMBER >= 0x10101000L)
 	SSL_CTX_sess_set_new_cb(tls->ctx, session_new_cb);
 	SSL_CTX_sess_set_remove_cb(tls->ctx, session_remove_cb);
 
 	return 0;
-#else
-	return EOPNOTSUPP;
-#endif
 }
 
 
@@ -1964,17 +1742,14 @@ SSL_CTX *tls_ssl_ctx(const struct tls *tls)
 }
 
 
-#if !defined(LIBRESSL_VERSION_NUMBER)
 static void tls_cert_destructor(void *arg)
 {
 	struct tls_cert *uc = arg;
 
 	mem_deref(uc->host);
-	X509_free(uc->x509);
-	EVP_PKEY_free(uc->pkey);
-	sk_X509_pop_free(uc->chain, X509_free);
+	if (uc->ctx)
+		SSL_CTX_free(uc->ctx);
 }
-#endif
 
 
 /**
@@ -1990,11 +1765,8 @@ static void tls_cert_destructor(void *arg)
  */
 int tls_add_certf(struct tls *tls, const char *certf, const char *host)
 {
-#if !defined(LIBRESSL_VERSION_NUMBER)
 	struct tls_cert *uc;
-	BIO *bio = NULL;
 	int err = 0;
-	int ret;
 
 	if (!tls || !certf)
 		return EINVAL;
@@ -2006,73 +1778,30 @@ int tls_add_certf(struct tls *tls, const char *certf, const char *host)
 	if (str_isset(host)) {
 		err = str_dup(&uc->host, host);
 		if (err)
-			goto out;
+			goto err;
 	}
 
-	bio = BIO_new_file(certf, "r");
-	if (!bio) {
-		err = EIO;
-		goto out;
+	err = tls_ctx_alloc(&uc->ctx, TLS_METHOD_TLS, certf, NULL, NULL);
+	if (err)
+		goto err;
+
+	X509_STORE *ca = SSL_CTX_get_cert_store(tls->ctx);
+	if (ca) {
+		X509_STORE_up_ref(ca);
+		SSL_CTX_set_cert_store(uc->ctx, ca);
 	}
 
-	uc->x509 = PEM_read_bio_X509(bio, NULL, NULL, NULL);
-	if (!uc->x509) {
-		DEBUG_WARNING("Can't read certificate from file: %s\n", certf);
-		err = ENOTSUP;
-		goto out;
-	}
+	list_append(&tls->certs, &uc->le, uc);
+	if (list_count(&tls->certs) == 1)
+		tls_enable_sni(tls);
 
-	while (1) {
-		X509 *ca = PEM_read_bio_X509(bio, NULL, 0, NULL);
-		if (!ca)
-			break;
+	return 0;
 
-		if (!uc->chain)
-			uc->chain = sk_X509_new_null();
-
-		if (!uc->chain) {
-			err = ENOMEM;
-			goto out;
-		}
-
-		if (!sk_X509_push(uc->chain, ca)) {
-			err = ENOMEM;
-			goto out;
-		}
-	}
-
-	ret = BIO_reset(bio);
-	if (ret < 0 || !bio) {
-		err = EIO;
-		goto out;
-	}
-
-	uc->pkey = PEM_read_bio_PrivateKey(bio, NULL, NULL, NULL);
-	if (!uc->pkey) {
-		DEBUG_WARNING("Can't read private key from file: %s\n", certf);
-		err = ENOTSUP;
-		goto out;
-	}
-
-out:
-	BIO_free(bio);
-	if (err) {
-		ERR_clear_error();
-		mem_deref(uc);
-	}
-	else {
-		list_append(&tls->certs, &uc->le, uc);
-		if (list_count(&tls->certs) == 1)
-			tls_enable_sni(tls);
-	}
+err:
+	ERR_clear_error();
+	mem_deref(uc);
 
 	return err;
-#else
-	(void)tls;
-	(void)certf;
-	(void)host;
-	return ENOSYS;
-#endif
 }
 
 
@@ -2085,36 +1814,15 @@ out:
  */
 X509 *tls_cert_x509(struct tls_cert *hc)
 {
-	return hc ? hc->x509 : NULL;
+	return hc ? SSL_CTX_get0_certificate(hc->ctx) : NULL;
 }
 
 
-/**
- * Returns the private key of the TLS certificate
- *
- * @param hc  TLS certificate
- *
- * @return The OpenSSL EVP_PKEY
- */
-EVP_PKEY *tls_cert_pkey(struct tls_cert *hc)
-{
-	return hc ? hc->pkey : NULL;
+SSL_CTX *tls_cert_ctx(struct tls_cert *hc) {
+
+	return hc ? hc->ctx : NULL;
 }
 
-
-/*
- * Returns the certificate chain of the TLS certificate
- *
- * @param hc  TLS certificate
- *
- * @return The OpenSSL stack of X509
- */
-struct stack_st_X509 *tls_cert_chain(struct tls_cert *hc)
-{
-	return hc ? hc->chain : NULL;
-}
-
-
 /**
  * Returns the host name of the TLS certificate
  *
@@ -2200,7 +1908,7 @@ int tls_verify_client_post_handshake(struct tls_conn *tc)
  *
  * @return 0 if success, otherwise errorcode
  */
-int tls_set_resumption(struct tls *tls, const enum tls_resume_mode mode)
+int tls_set_resumption(struct tls *tls, enum tls_resume_mode mode)
 {
 	long ok = 1;
 
@@ -2232,3 +1940,154 @@ int tls_set_resumption(struct tls *tls, const enum tls_resume_mode mode)
 
 	return 0;
 }
+
+
+/**
+ * Change used certificate+key of an existing SSL object
+ *
+ * @param tls       TLS Object
+ * @param chain     Cert (chain) + Key in PEM format
+ * @param len_chain Length of certificate + key PEM string
+ *
+ * @return int 0 if success, otherwise errorcode
+ */
+int tls_set_certificate_chain_pem(struct tls *tls, const char *chain,
+				  size_t len_chain)
+{
+	STACK_OF(X509) *cert_stack = NULL;
+	BIO *bio_mem = NULL;
+	EVP_PKEY *pkey = NULL;
+	X509 *leaf_cert = NULL;
+	int err = ENOMEM;
+
+	if (!tls || !chain || !len_chain)
+		return EINVAL;
+
+	bio_mem = BIO_new_mem_buf(chain, (int)len_chain);
+	cert_stack = sk_X509_new_null();
+	if (!bio_mem || !cert_stack)
+		goto out;
+
+	X509 *cert;
+	while ((cert = PEM_read_bio_X509(bio_mem, NULL, NULL, NULL)) != NULL) {
+		int n = sk_X509_push(cert_stack, cert);
+		if (n < 1) {
+			X509_free(cert);
+			goto out;
+		}
+	}
+
+	err = EINVAL;
+
+	if (sk_X509_num(cert_stack) == 0)
+		goto out;
+
+	leaf_cert = sk_X509_shift(cert_stack);
+	long ok = SSL_CTX_use_certificate(tls->ctx, leaf_cert);
+	if (ok <= 0) {
+		X509_free(leaf_cert);
+		goto out;
+	}
+
+	if (sk_X509_num(cert_stack)) {
+		ok = SSL_CTX_clear_chain_certs(tls->ctx);
+		if (!ok)
+			goto out;
+
+		while((cert = sk_X509_shift(cert_stack)) != NULL){
+			ok = SSL_CTX_add0_chain_cert(tls->ctx, cert);
+			if (!ok) {
+				X509_free(cert);
+				goto out;
+			}
+		}
+	}
+
+	BIO_free(bio_mem);
+	bio_mem = BIO_new_mem_buf(chain, (int)len_chain);
+	if (!bio_mem) {
+		err = ENOMEM;
+		goto out;
+	}
+
+	pkey = PEM_read_bio_PrivateKey(bio_mem, NULL, NULL, NULL);
+	if (!pkey)
+		goto out;
+
+	ok = SSL_CTX_use_PrivateKey(tls->ctx, pkey);
+	if (ok <= 0) {
+		err = EKEYREJECTED;
+		goto out;
+	}
+
+	ok = SSL_CTX_check_private_key(tls->ctx);
+	if (ok <= 0)
+		goto out;
+
+	if (tls->cert)
+		X509_free(tls->cert);
+
+	tls->cert = leaf_cert;
+	leaf_cert = NULL;
+
+	err = 0;
+
+out:
+	if (bio_mem)
+		BIO_free(bio_mem);
+	if (leaf_cert)
+		X509_free(leaf_cert);
+	if (cert_stack)
+		sk_X509_pop_free(cert_stack, X509_free);
+	if (pkey)
+		EVP_PKEY_free(pkey);
+	if (err)
+		ERR_clear_error();
+
+	return err;
+}
+
+
+/**
+ * Change used certificate+key of an existing SSL object
+ *
+ * @param tls  TLS Object
+ * @param path Path to Cert (chain) + Key file (PEM format)
+ *
+ * @return int 0 if success, otherwise errorcode
+ */
+int tls_set_certificate_chain(struct tls *tls, const char *path)
+{
+	X509 *cert;
+	int ok = 0;
+
+	if (!tls || !path)
+		return EINVAL;
+
+	ok = SSL_CTX_use_certificate_chain_file(tls->ctx, path);
+	if (ok <= 0) {
+		ERR_clear_error();
+		return ENOENT;
+	}
+
+	ok = SSL_CTX_use_PrivateKey_file(tls->ctx, path, SSL_FILETYPE_PEM);
+	if (ok <= 0) {
+		ERR_clear_error();
+		return EKEYREJECTED;
+	}
+
+	cert = SSL_CTX_get0_certificate(tls->ctx);
+	if (!cert) {
+		ERR_clear_error();
+		return ENOENT;
+	}
+
+	X509_up_ref(cert);
+
+	if (tls->cert)
+		X509_free(tls->cert);
+
+	tls->cert = cert;
+
+	return 0;
+}
