@@ -52,15 +52,14 @@ static const struct test tests[] = {
 	TEST(test_aac),
 	TEST(test_aes),
 	TEST(test_aes_gcm),
+	TEST(test_async),
 	TEST(test_au),
 	TEST(test_aubuf),
 	TEST(test_aulength),
 	TEST(test_aulevel),
 	TEST(test_auposition),
 	TEST(test_auresamp),
-	TEST(test_async),
 	TEST(test_av1),
-	TEST(test_dd),
 	TEST(test_base64),
 	TEST(test_bfcp),
 	TEST(test_bfcp_bin),
@@ -69,9 +68,10 @@ static const struct test tests[] = {
 	TEST(test_btrace),
 	TEST(test_conf),
 	TEST(test_crc32),
+	TEST(test_dd),
+	TEST(test_dns_dname),
 	TEST(test_dns_hdr),
 	TEST(test_dns_rr),
-	TEST(test_dns_dname),
 	TEST(test_dsp),
 #ifdef USE_TLS
 	TEST(test_dtls),
@@ -105,18 +105,18 @@ static const struct test tests[] = {
 	TEST(test_g711_alaw),
 	TEST(test_g711_ulaw),
 	TEST(test_h264),
-	TEST(test_h264_sps),
 	TEST(test_h264_packet),
+	TEST(test_h264_sps),
 	TEST(test_h265),
 	TEST(test_h265_packet),
 	TEST(test_hash),
 	TEST(test_hmac_sha1),
 	TEST(test_hmac_sha256),
 	TEST(test_http),
-	TEST(test_http_loop),
-	TEST(test_http_large_body),
 	TEST(test_http_conn),
 	TEST(test_http_conn_large_body),
+	TEST(test_http_large_body),
+	TEST(test_http_loop),
 #ifdef USE_TLS
 	TEST(test_https_loop),
 	TEST(test_http_client_set_tls),
@@ -196,37 +196,37 @@ static const struct test tests[] = {
 #ifdef USE_TLS
 	TEST(test_sip_transp_add_client_cert),
 #endif
+	TEST(test_fmt_trim),
 	TEST(test_sipevent),
 	TEST(test_sipsess),
-	TEST(test_sipsess_reject),
-	TEST(test_sipsess_blind_transfer),
-	TEST(test_sipsess_100rel_caller_require),
-	TEST(test_sipsess_100rel_supported),
-	TEST(test_sipsess_100rel_answer_not_allowed),
 	TEST(test_sipsess_100rel_420),
 	TEST(test_sipsess_100rel_421),
+	TEST(test_sipsess_100rel_answer_not_allowed),
+	TEST(test_sipsess_100rel_caller_require),
+	TEST(test_sipsess_100rel_supported),
+	TEST(test_sipsess_blind_transfer),
+	TEST(test_sipsess_reject),
+	TEST(test_sipsess_update_no_sdp),
 	TEST(test_sipsess_update_uac),
 	TEST(test_sipsess_update_uas),
-	TEST(test_sipsess_update_no_sdp),
-	TEST(test_srtp),
 	TEST(test_srtcp),
-	TEST(test_srtp_gcm),
 	TEST(test_srtcp_gcm),
-	TEST(test_stun_req),
-	TEST(test_stun_resp),
-	TEST(test_stun_reqltc),
+	TEST(test_srtp),
+	TEST(test_srtp_gcm),
 	TEST(test_stun),
+	TEST(test_stun_req),
+	TEST(test_stun_reqltc),
+	TEST(test_stun_resp),
 	TEST(test_sys_endian),
-	TEST(test_sys_rand),
+	TEST(test_sys_fs_fopen),
 	TEST(test_sys_fs_isdir),
 	TEST(test_sys_fs_isfile),
-	TEST(test_sys_fs_fopen),
 	TEST(test_sys_getenv),
+	TEST(test_sys_rand),
 	TEST(test_tcp),
 	TEST(test_tcp_tos),
 	TEST(test_telev),
 	TEST(test_text2pcap),
-	TEST(test_fmt_trim),
 #ifdef USE_TLS
 	TEST(test_tls),
 	TEST(test_tls_ec),
@@ -237,6 +237,9 @@ static const struct test tests[] = {
 	TEST(test_tls_session_reuse_tls_v12),
 	TEST(test_tls_sni),
 #endif
+	TEST(test_thread),
+	TEST(test_thread_tss),
+	TEST(test_trace),
 	TEST(test_trice_cand),
 	TEST(test_trice_candpair),
 	TEST(test_trice_checklist),
@@ -249,18 +252,15 @@ static const struct test tests[] = {
 	TEST(test_unixsock),
 	TEST(test_uri),
 	TEST(test_uri_encode),
-	TEST(test_uri_headers),
-	TEST(test_uri_user),
-	TEST(test_uri_params_headers),
 	TEST(test_uri_escape),
+	TEST(test_uri_headers),
+	TEST(test_uri_params_headers),
+	TEST(test_uri_user),
 	TEST(test_vid),
 	TEST(test_vidconv),
-	TEST(test_vidconv_scaling),
 	TEST(test_vidconv_pixel_formats),
+	TEST(test_vidconv_scaling),
 	TEST(test_websock),
-	TEST(test_trace),
-	TEST(test_thread),
-	TEST(test_thread_tss),
 
 #ifdef USE_TLS
 	/* combination tests: */
@@ -273,6 +273,7 @@ static const struct test tests_integration[] = {
 	TEST(test_dns_cache_http_integration),
 	TEST(test_dns_http_integration),
 	TEST(test_dns_integration),
+	TEST(test_dns_nameservers),
 	TEST(test_net_dst_source_addr_get),
 	TEST(test_rtp_listen),
 	TEST(test_sip_drequestf_network),
@@ -316,8 +317,13 @@ static void hide_output(void)
 #endif
 
 	int fd_out = open("stdout.out", O_WRONLY | O_CREAT, mode);
-	int fd_err = open("stderr.out", O_WRONLY | O_CREAT, mode);
+	if (fd_out < 0)
+		return;
 	(void)dup2(fd_out, fileno(stdout));
+
+	int fd_err = open("stderr.out", O_WRONLY | O_CREAT, mode);
+	if (fd_err < 0)
+		return;
 	(void)dup2(fd_err, fileno(stderr));
 }
 
