@@ -178,6 +178,7 @@ int test_dns_hdr(void);
 int test_dns_integration(void);
 int test_dns_rr(void);
 int test_dns_dname(void);
+int test_dns_nameservers(void);
 int test_dsp(void);
 int test_dtmf(void);
 int test_fir(void);
@@ -534,7 +535,7 @@ struct dns_server {
 	bool rotate;
 };
 
-int dns_server_alloc(struct dns_server **srvp, bool rotate);
+int dns_server_alloc(struct dns_server **srvp, const char *laddr, bool rotate);
 int dns_server_add_a(struct dns_server *srv, const char *name, uint32_t addr,
 		     int64_t ttl);
 int dns_server_add_aaaa(struct dns_server *srv, const char *name,
