@@ -283,8 +283,7 @@ static void ice_failed_handler2(int err, uint16_t scode,
 static int fixture_init(struct fixture *f)
 {
 	const struct trice_conf conf = {
-		.debug = DEBUG,
-		.enable_prflx = true
+		.debug = DEBUG
 	};
 	int err;
 
@@ -715,7 +714,7 @@ static int candpair_add_1_local_and_1_remote_candidate_and_create_pair(void)
 
 	ADD_REMOTE_HOST_CANDIDATE(&addr);
 
-	/* the checklist is formated automatically */
+	/* the checklist is formatted automatically */
 
 	TEST_EQUALS(1, list_count(trice_lcandl(f->icem)));
 	TEST_EQUALS(1, list_count(trice_rcandl(f->icem)));
@@ -917,6 +916,7 @@ static int exchange_candidates(struct trice *dst, const struct trice *src)
 static int checklist_tcp_simple(enum ice_tcptype tcptype)
 {
 	struct le *le;
+	char *buf = NULL;
 	FIXTURE_INIT;
 
 #if 0
@@ -954,9 +954,9 @@ static int checklist_tcp_simple(enum ice_tcptype tcptype)
 
 	TEST_ERR(f->err);
 
-#if 0
-	re_printf("\n\n%H\n", trice_debug, f->icem);
-#endif
+	err = re_sdprintf(&buf, "%H", trice_debug, f->icem);
+	TEST_ERR(err);
+	ASSERT_TRUE(str_isset(buf));
 
 	TEST_ASSERT(f->n_estabh > 0);
 
@@ -984,6 +984,7 @@ static int checklist_tcp_simple(enum ice_tcptype tcptype)
 
  out:
 	fixture_close(f);
+	mem_deref(buf);
 	return err;
 }
 
@@ -1048,6 +1049,7 @@ static int checklist_udp_loop(bool fw_a, bool fw_b)
 	struct ice_lcand *lcand, *lcand2;
 	struct sa laddr2;
 	uint32_t prio;
+	char *buf = NULL;
 	FIXTURE_INIT;
 
 #if 0
@@ -1118,10 +1120,9 @@ static int checklist_udp_loop(bool fw_a, bool fw_b)
 
 	TEST_ERR(f->err);
 
-#if 0
-	re_printf("\nA:\n%H\n", trice_debug, f->icem);
-	re_printf("\nB:\n%H\n", trice_debug, f->icem2);
-#endif
+	err = re_sdprintf(&buf, "%H", trice_debug, f->icem);
+	TEST_ERR(err);
+	ASSERT_TRUE(str_isset(buf));
 
 	TEST_ASSERT(f->n_estabh > 0);
 
@@ -1135,6 +1136,7 @@ static int checklist_udp_loop(bool fw_a, bool fw_b)
 
  out:
 	fixture_close(f);
+	mem_deref(buf);
 	return err;
 }
 
